@@ -1,5 +1,5 @@
 # Licensed under the LGPL: https://www.gnu.org/licenses/old-licenses/lgpl-2.1.en.html
-# For details: https://github.com/PyCQA/astroid/blob/master/COPYING.LESSER
+# For details: https://github.com/PyCQA/astroid/blob/master/LICENSE
 """
 Astroid hook for the attrs library
 
@@ -10,7 +10,6 @@ for attrs classes
 import astroid
 from astroid import MANAGER
 
-
 ATTRIB_NAMES = frozenset(("attr.ib", "attrib", "attr.attrib"))
 ATTRS_NAMES = frozenset(("attr.s", "attrs", "attr.attrs", "attr.attributes"))
 
@@ -37,14 +36,19 @@ def attr_attributes_transform(node):
     node.locals["__attrs_attrs__"] = [astroid.Unknown(parent=node)]
 
     for cdefbodynode in node.body:
-        if not isinstance(cdefbodynode, astroid.Assign):
+        if not isinstance(cdefbodynode, (astroid.Assign, astroid.AnnAssign)):
             continue
         if isinstance(cdefbodynode.value, astroid.Call):
             if cdefbodynode.value.func.as_string() not in ATTRIB_NAMES:
                 continue
         else:
             continue
-        for target in cdefbodynode.targets:
+        targets = (
+            cdefbodynode.targets
+            if hasattr(cdefbodynode, "targets")
+            else [cdefbodynode.target]
+        )
+        for target in targets:
 
             rhs_node = astroid.Unknown(
                 lineno=cdefbodynode.lineno,
@@ -52,6 +56,7 @@ def attr_attributes_transform(node):
                 parent=cdefbodynode,
             )
             node.locals[target.name] = [rhs_node]
+            node.instance_attrs[target.name] = [rhs_node]
 
 
 MANAGER.register_transform(
