@@ -1,7 +1,6 @@
-# -*- coding: utf-8 -*-
 # Copyright (c) 2012-2015 LOGILAB S.A. (Paris, FRANCE) <contact@logilab.fr>
 # Copyright (c) 2013-2014 Google, Inc.
-# Copyright (c) 2014-2018 Claudiu Popa <pcmanticore@gmail.com>
+# Copyright (c) 2014-2020 Claudiu Popa <pcmanticore@gmail.com>
 # Copyright (c) 2014 Eevee (Alex Munroe) <amunroe@yelp.com>
 # Copyright (c) 2015-2016 Ceridwen <ceridwenv@gmail.com>
 # Copyright (c) 2015 Dmitry Pribysh <dmand@yandex.ru>
@@ -11,9 +10,17 @@
 # Copyright (c) 2016 Mateusz Bysiek <mb@mbdev.pl>
 # Copyright (c) 2017 Hugo <hugovk@users.noreply.github.com>
 # Copyright (c) 2017 Łukasz Rogalski <rogalski.91@gmail.com>
+# Copyright (c) 2018 Ville Skyttä <ville.skytta@iki.fi>
+# Copyright (c) 2019 Ashley Whetter <ashley@awhetter.co.uk>
+# Copyright (c) 2020 hippo91 <guillaume.peillex@gmail.com>
+# Copyright (c) 2020 Ram Rachum <ram@rachum.com>
+# Copyright (c) 2021 Andrew Haigh <hello@nelf.in>
+# Copyright (c) 2021 pre-commit-ci[bot] <bot@noreply.github.com>
+# Copyright (c) 2021 Pierre Sassoulas <pierre.sassoulas@gmail.com>
+# Copyright (c) 2021 Marc Mueller <30130371+cdce8p@users.noreply.github.com>
 
 # Licensed under the LGPL: https://www.gnu.org/licenses/old-licenses/lgpl-2.1.en.html
-# For details: https://github.com/PyCQA/astroid/blob/master/COPYING.LESSER
+# For details: https://github.com/PyCQA/astroid/blob/master/LICENSE
 
 """Astroid hooks for the Python standard library."""
 
@@ -21,13 +28,19 @@ import functools
 import keyword
 from textwrap import dedent
 
-from astroid import MANAGER, UseInferenceDefault, inference_tip, InferenceError
-from astroid import arguments
-from astroid import exceptions
-from astroid import nodes
+from astroid import (
+    MANAGER,
+    AstroidTypeError,
+    AstroidValueError,
+    InferenceError,
+    UseInferenceDefault,
+    arguments,
+    exceptions,
+    inference_tip,
+    nodes,
+    util,
+)
 from astroid.builder import AstroidBuilder, extract_node
-from astroid import util
-
 
 TYPING_NAMEDTUPLE_BASENAMES = {"NamedTuple", "typing.NamedTuple"}
 ENUM_BASE_NAMES = {
@@ -47,15 +60,15 @@ def _infer_first(node, context):
         value = next(node.infer(context=context))
         if value is util.Uninferable:
             raise UseInferenceDefault()
-        else:
-            return value
-    except StopIteration:
-        raise InferenceError()
+        return value
+    except StopIteration as exc:
+        raise InferenceError from exc
 
 
 def _find_func_form_arguments(node, context):
-    def _extract_namedtuple_arg_or_keyword(position, key_name=None):
-
+    def _extract_namedtuple_arg_or_keyword(  # pylint: disable=inconsistent-return-statements
+        position, key_name=None
+    ):
         if len(args) > position:
             return _infer_first(args[position], context)
         if key_name and key_name in found_keywords:
@@ -76,7 +89,7 @@ def _find_func_form_arguments(node, context):
 
 
 def infer_func_form(node, base_type, context=None, enum=False):
-    """Specific inference function for namedtuple or Python 3 enum. """
+    """Specific inference function for namedtuple or Python 3 enum."""
     # node is a Call node, class name as first argument and generated class
     # attributes as second argument
 
@@ -86,7 +99,7 @@ def infer_func_form(node, base_type, context=None, enum=False):
         name, names = _find_func_form_arguments(node, context)
         try:
             attributes = names.value.replace(",", " ").split()
-        except AttributeError:
+        except AttributeError as exc:
             if not enum:
                 attributes = [
                     _infer_first(const, context).value for const in names.elts
@@ -115,11 +128,18 @@ def infer_func_form(node, base_type, context=None, enum=False):
                             _infer_first(const, context).value for const in names.elts
                         ]
                 else:
-                    raise AttributeError
+                    raise AttributeError from exc
                 if not attributes:
-                    raise AttributeError
-    except (AttributeError, exceptions.InferenceError):
-        raise UseInferenceDefault()
+                    raise AttributeError from exc
+    except (AttributeError, exceptions.InferenceError) as exc:
+        raise UseInferenceDefault from exc
+
+    if not enum:
+        # namedtuple maps sys.intern(str()) over over field_names
+        attributes = [str(attr) for attr in attributes]
+        # XXX this should succeed *unless* __str__/__repr__ is incorrect or throws
+        # in which case we should not have inferred these values and raised earlier
+    attributes = [attr for attr in attributes if " " not in attr]
 
     # If we can't infer the name of the class, don't crash, up to this point
     # we know it is a namedtuple anyway.
@@ -167,17 +187,21 @@ def infer_named_tuple(node, context=None):
     class_node, name, attributes = infer_func_form(
         node, tuple_base_name, context=context
     )
-    call_site = arguments.CallSite.from_call(node)
+    call_site = arguments.CallSite.from_call(node, context=context)
     func = next(extract_node("import collections; collections.namedtuple").infer())
     try:
         rename = next(call_site.infer_argument(func, "rename", context)).bool_value()
     except InferenceError:
         rename = False
 
-    if rename:
-        attributes = _get_renamed_namedtuple_attributes(attributes)
+    try:
+        attributes = _check_namedtuple_attributes(name, attributes, rename)
+    except AstroidTypeError as exc:
+        raise UseInferenceDefault("TypeError: " + str(exc)) from exc
+    except AstroidValueError as exc:
+        raise UseInferenceDefault("ValueError: " + str(exc)) from exc
 
-    replace_args = ", ".join("{arg}=None".format(arg=arg) for arg in attributes)
+    replace_args = ", ".join(f"{arg}=None" for arg in attributes)
     field_def = (
         "    {name} = property(lambda self: self[{index:d}], "
         "doc='Alias for field number {index:d}')"
@@ -236,8 +260,41 @@ def _get_renamed_namedtuple_attributes(field_names):
     return tuple(names)
 
 
+def _check_namedtuple_attributes(typename, attributes, rename=False):
+    attributes = tuple(attributes)
+    if rename:
+        attributes = _get_renamed_namedtuple_attributes(attributes)
+
+    # The following snippet is derived from the CPython Lib/collections/__init__.py sources
+    # <snippet>
+    for name in (typename,) + attributes:
+        if not isinstance(name, str):
+            raise AstroidTypeError("Type names and field names must be strings")
+        if not name.isidentifier():
+            raise AstroidValueError(
+                "Type names and field names must be valid" + f"identifiers: {name!r}"
+            )
+        if keyword.iskeyword(name):
+            raise AstroidValueError(
+                f"Type names and field names cannot be a keyword: {name!r}"
+            )
+
+    seen = set()
+    for name in attributes:
+        if name.startswith("_") and not rename:
+            raise AstroidValueError(
+                f"Field names cannot start with an underscore: {name!r}"
+            )
+        if name in seen:
+            raise AstroidValueError(f"Encountered duplicate field name: {name!r}")
+        seen.add(name)
+    # </snippet>
+
+    return attributes
+
+
 def infer_enum(node, context=None):
-    """ Specific inference function for enum Call node. """
+    """Specific inference function for enum Call node."""
     enum_meta = extract_node(
         """
     class EnumMeta(object):
@@ -295,7 +352,7 @@ INT_FLAG_ADDITION_METHODS = """
 
 
 def infer_enum_class(node):
-    """ Specific inference for enums. """
+    """Specific inference for enums."""
     for basename in node.basenames:
         # TODO: doesn't handle subclasses yet. This implementation
         # is a hack to support enums.
@@ -304,11 +361,11 @@ def infer_enum_class(node):
         if node.root().name == "enum":
             # Skip if the class is directly from enum module.
             break
+        dunder_members = {}
         for local, values in node.locals.items():
             if any(not isinstance(value, nodes.AssignName) for value in values):
                 continue
 
-            targets = []
             stmt = values[0].statement()
             if isinstance(stmt, nodes.Assign):
                 if isinstance(stmt.targets[0], nodes.Tuple):
@@ -317,6 +374,8 @@ def infer_enum_class(node):
                     targets = stmt.targets
             elif isinstance(stmt, nodes.AnnAssign):
                 targets = [stmt.target]
+            else:
+                continue
 
             inferred_return_value = None
             if isinstance(stmt, nodes.Assign):
@@ -330,6 +389,8 @@ def infer_enum_class(node):
 
             new_targets = []
             for target in targets:
+                if isinstance(target, nodes.Starred):
+                    continue
                 # Replace all the assignments with our mocked class.
                 classdef = dedent(
                     """
@@ -358,7 +419,16 @@ def infer_enum_class(node):
                 for method in node.mymethods():
                     fake.locals[method.name] = [method]
                 new_targets.append(fake.instantiate_class())
+                dunder_members[local] = fake
             node.locals[local] = new_targets
+        members = nodes.Dict(parent=node)
+        members.postinit(
+            [
+                (nodes.Const(k, parent=members), nodes.Name(v.name, parent=members))
+                for k, v in dunder_members.items()
+            ]
+        )
+        node.locals["__members__"] = [members]
         break
     return node
 
@@ -381,17 +451,41 @@ def infer_typing_namedtuple_class(class_node, context=None):
     generated_class_node = next(infer_named_tuple(node, context))
     for method in class_node.mymethods():
         generated_class_node.locals[method.name] = [method]
+
+    for assign in class_node.body:
+        if not isinstance(assign, nodes.Assign):
+            continue
+
+        for target in assign.targets:
+            attr = target.name
+            generated_class_node.locals[attr] = class_node.locals[attr]
+
     return iter((generated_class_node,))
 
 
+def infer_typing_namedtuple_function(node, context=None):
+    """
+    Starting with python3.9, NamedTuple is a function of the typing module.
+    The class NamedTuple is build dynamically through a call to `type` during
+    initialization of the `_NamedTuple` variable.
+    """
+    klass = extract_node(
+        """
+        from typing import _NamedTuple
+        _NamedTuple
+        """
+    )
+    return klass.infer(context)
+
+
 def infer_typing_namedtuple(node, context=None):
     """Infer a typing.NamedTuple(...) call."""
     # This is essentially a namedtuple with different arguments
     # so we extract the args and infer a named tuple.
     try:
         func = next(node.func.infer())
-    except InferenceError:
-        raise UseInferenceDefault
+    except InferenceError as exc:
+        raise UseInferenceDefault from exc
 
     if func.qname() != "typing.NamedTuple":
         raise UseInferenceDefault
@@ -415,9 +509,7 @@ def infer_typing_namedtuple(node, context=None):
         field_names = "({},)".format(",".join(names))
     else:
         field_names = "''"
-    node = extract_node(
-        "namedtuple({typename}, {fields})".format(typename=typename, fields=field_names)
-    )
+    node = extract_node(f"namedtuple({typename}, {field_names})")
     return infer_named_tuple(node, context)
 
 
@@ -435,6 +527,12 @@ MANAGER.register_transform(
 MANAGER.register_transform(
     nodes.ClassDef, inference_tip(infer_typing_namedtuple_class), _has_namedtuple_base
 )
+MANAGER.register_transform(
+    nodes.FunctionDef,
+    inference_tip(infer_typing_namedtuple_function),
+    lambda node: node.name == "NamedTuple"
+    and getattr(node.root(), "name", None) == "typing",
+)
 MANAGER.register_transform(
     nodes.Call, inference_tip(infer_typing_namedtuple), _looks_like_typing_namedtuple
 )
