@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 # Copyright (c) 2009-2014 LOGILAB S.A. (Paris, FRANCE) <contact@logilab.fr>
 # Copyright (c) 2013-2020 Claudiu Popa <pcmanticore@gmail.com>
 # Copyright (c) 2013-2014 Google, Inc.
@@ -15,24 +14,31 @@
 # Copyright (c) 2018 Serhiy Storchaka <storchaka@gmail.com>
 # Copyright (c) 2018 Nick Drozd <nicholasdrozd@gmail.com>
 # Copyright (c) 2018 Bryce Guinta <bryce.paul.guinta@gmail.com>
-# Copyright (c) 2019-2020 Ashley Whetter <ashley@awhetter.co.uk>
+# Copyright (c) 2019-2021 Ashley Whetter <ashley@awhetter.co.uk>
 # Copyright (c) 2019 Hugo van Kemenade <hugovk@users.noreply.github.com>
 # Copyright (c) 2019 Zbigniew Jędrzejewski-Szmek <zbyszek@in.waw.pl>
+# Copyright (c) 2021 Marc Mueller <30130371+cdce8p@users.noreply.github.com>
+# Copyright (c) 2021 Federico Bond <federicobond@gmail.com>
+# Copyright (c) 2021 Pierre Sassoulas <pierre.sassoulas@gmail.com>
+# Copyright (c) 2021 hippo91 <guillaume.peillex@gmail.com>
 
 # Licensed under the LGPL: https://www.gnu.org/licenses/old-licenses/lgpl-2.1.en.html
-# For details: https://github.com/PyCQA/astroid/blob/master/COPYING.LESSER
+# For details: https://github.com/PyCQA/astroid/blob/master/LICENSE
 
 """this module contains utilities for rebuilding an _ast tree in
 order to get a single Astroid representation
 """
 
 import sys
-from typing import Optional
+from typing import TYPE_CHECKING, Optional
 
 import astroid
-from astroid._ast import parse_function_type_comment, get_parser_module, ParserModule
 from astroid import nodes
+from astroid._ast import ParserModule, get_parser_module, parse_function_type_comment
+from astroid.node_classes import NodeNG
 
+if TYPE_CHECKING:
+    import ast
 
 CONST_NAME_TRANSFORMS = {"None": None, "True": True, "False": False}
 
@@ -43,6 +49,7 @@ REDIRECT = {
     "GenExprFor": "Comprehension",
     "excepthandler": "ExceptHandler",
     "keyword": "Keyword",
+    "match_case": "MatchCase",
 }
 PY37 = sys.version_info >= (3, 7)
 PY38 = sys.version_info >= (3, 8)
@@ -238,7 +245,7 @@ class TreeRebuilder:
 
         return type_object.value
 
-    def check_function_type_comment(self, node):
+    def check_function_type_comment(self, node, parent):
         type_comment = getattr(node, "type_comment", None)
         if not type_comment:
             return None
@@ -251,10 +258,10 @@ class TreeRebuilder:
 
         returns = None
         argtypes = [
-            self.visit(elem, node) for elem in (type_comment_ast.argtypes or [])
+            self.visit(elem, parent) for elem in (type_comment_ast.argtypes or [])
         ]
         if type_comment_ast.returns:
-            returns = self.visit(type_comment_ast.returns, node)
+            returns = self.visit(type_comment_ast.returns, parent)
 
         return returns, argtypes
 
@@ -425,6 +432,7 @@ class TreeRebuilder:
             getattr(node, "lineno", None),
             getattr(node, "col_offset", None),
             parent,
+            getattr(node, "kind", None),
         )
 
     def visit_continue(self, node, parent):
@@ -550,7 +558,7 @@ class TreeRebuilder:
         )
         return newnode
 
-    # Not used in Python 3.8+.
+    # Not used in Python 3.9+.
     def visit_extslice(self, node, parent):
         """visit an ExtSlice node by returning a fresh instance of it"""
         newnode = nodes.ExtSlice(parent=parent)
@@ -615,7 +623,7 @@ class TreeRebuilder:
             returns = None
 
         type_comment_args = type_comment_returns = None
-        type_comment_annotation = self.check_function_type_comment(node)
+        type_comment_annotation = self.check_function_type_comment(node, newnode)
         if type_comment_annotation:
             type_comment_returns, type_comment_args = type_comment_annotation
         newnode.postinit(
@@ -727,7 +735,7 @@ class TreeRebuilder:
         )
         return newnode
 
-    # Not used in Python 3.8+.
+    # Not used in Python 3.9+.
     def visit_index(self, node, parent):
         """visit a Index node by returning a fresh instance of it"""
         newnode = nodes.Index(parent=parent)
@@ -814,6 +822,7 @@ class TreeRebuilder:
             getattr(node, "lineno", None),
             getattr(node, "col_offset", None),
             parent,
+            getattr(node, "kind", None),
         )
 
     # Not used in Python 3.8+.
@@ -1008,3 +1017,88 @@ class TreeRebuilder:
         if node.value is not None:
             newnode.postinit(self.visit(node.value, newnode))
         return newnode
+
+    def visit_match(self, node: "ast.Match", parent: NodeNG) -> nodes.Match:
+        newnode = nodes.Match(node.lineno, node.col_offset, parent)
+        newnode.postinit(
+            subject=self.visit(node.subject, newnode),
+            cases=[self.visit(case, newnode) for case in node.cases],
+        )
+        return newnode
+
+    def visit_matchcase(
+        self, node: "ast.match_case", parent: NodeNG
+    ) -> nodes.MatchCase:
+        newnode = nodes.MatchCase(parent=parent)
+        newnode.postinit(
+            pattern=self.visit(node.pattern, newnode),
+            guard=_visit_or_none(node, "guard", self, newnode),
+            body=[self.visit(child, newnode) for child in node.body],
+        )
+        return newnode
+
+    def visit_matchvalue(
+        self, node: "ast.MatchValue", parent: NodeNG
+    ) -> nodes.MatchValue:
+        newnode = nodes.MatchValue(node.lineno, node.col_offset, parent)
+        newnode.postinit(value=self.visit(node.value, newnode))
+        return newnode
+
+    def visit_matchsingleton(
+        self, node: "ast.MatchSingleton", parent: NodeNG
+    ) -> nodes.MatchSingleton:
+        return nodes.MatchSingleton(
+            node.lineno, node.col_offset, parent, value=node.value
+        )
+
+    def visit_matchsequence(
+        self, node: "ast.MatchSequence", parent: NodeNG
+    ) -> nodes.MatchSequence:
+        newnode = nodes.MatchSequence(node.lineno, node.col_offset, parent)
+        newnode.postinit(
+            patterns=[self.visit(pattern, newnode) for pattern in node.patterns]
+        )
+        return newnode
+
+    def visit_matchmapping(
+        self, node: "ast.MatchMapping", parent: NodeNG
+    ) -> nodes.MatchMapping:
+        newnode = nodes.MatchMapping(node.lineno, node.col_offset, parent)
+        newnode.postinit(
+            keys=[self.visit(child, newnode) for child in node.keys],
+            patterns=[self.visit(pattern, newnode) for pattern in node.patterns],
+            rest=node.rest,
+        )
+        return newnode
+
+    def visit_matchclass(
+        self, node: "ast.MatchClass", parent: NodeNG
+    ) -> nodes.MatchClass:
+        newnode = nodes.MatchClass(node.lineno, node.col_offset, parent)
+        newnode.postinit(
+            cls=self.visit(node.cls, newnode),
+            patterns=[self.visit(pattern, newnode) for pattern in node.patterns],
+            kwd_attrs=node.kwd_attrs,
+            kwd_patterns=[
+                self.visit(pattern, newnode) for pattern in node.kwd_patterns
+            ],
+        )
+        return newnode
+
+    def visit_matchstar(self, node: "ast.MatchStar", parent: NodeNG) -> nodes.MatchStar:
+        return nodes.MatchStar(node.lineno, node.col_offset, parent, name=node.name)
+
+    def visit_matchas(self, node: "ast.MatchAs", parent: NodeNG) -> nodes.MatchAs:
+        newnode = nodes.MatchAs(None, None, parent)
+        newnode.postinit(
+            pattern=_visit_or_none(node, "pattern", self, newnode),
+            name=node.name,
+        )
+        return newnode
+
+    def visit_matchor(self, node: "ast.MatchOr", parent: NodeNG) -> nodes.MatchOr:
+        newnode = nodes.MatchOr(None, None, parent)
+        newnode.postinit(
+            patterns=[self.visit(pattern, newnode) for pattern in node.patterns]
+        )
+        return newnode
